@@ -11,8 +11,9 @@
 #ifndef _HOSTTABLE_H_
 #define _HOSTTABLE_H_
 
-#include <pthread.h>
-#include <tcsd_wrap.h>
+#include "rpc_tcstp.h"
+#include "threads.h"
+
 
 #define CONNECTION_TYPE_TCP_PERSISTANT	1
 
@@ -24,18 +25,19 @@ struct host_table_entry {
 	int type;
 	int socket;
 	struct tcsd_comm_data comm;
-	pthread_mutex_t lock; /* per-entry lock making calls serialized per-TCS */
+	MUTEX_DECLARE(lock);
 };
 
 struct host_table {
 	struct host_table_entry *entries;
-	pthread_mutex_t lock; /* Lock for the integrity of the host table list */
+	MUTEX_DECLARE(lock);
 };
 
-struct host_table_entry *get_table_entry(TSS_HCONTEXT);
+extern struct host_table *ht;
+struct host_table_entry *get_table_entry(TCS_CONTEXT_HANDLE);
 void put_table_entry(struct host_table_entry *);
 TSS_RESULT add_table_entry(TSS_HCONTEXT, BYTE *, int, struct host_table_entry **);
-void remove_table_entry(TSS_HCONTEXT);
+void remove_table_entry(TCS_CONTEXT_HANDLE);
 
 
 #endif
