@@ -24,7 +24,7 @@ struct event_wrapper {
 };
 
 struct event_log {
-	pthread_mutex_t lock;
+	MUTEX_DECLARE(lock);
 	struct ext_log_source *firmware_source;
 	struct ext_log_source *kernel_source;
 	struct event_wrapper **lists;
@@ -46,4 +46,17 @@ struct event_log {
 #define EVLOG_BIOS_SOURCE	NULL
 #endif
 
+
+TSS_RESULT event_log_init();
+TSS_RESULT event_log_final();
+TSS_RESULT copy_pcr_event(TSS_PCR_EVENT *, TSS_PCR_EVENT *);
+TSS_RESULT event_log_add(TSS_PCR_EVENT *, UINT32 *);
+TSS_PCR_EVENT *get_pcr_event(UINT32, UINT32);
+UINT32 get_num_events(UINT32);
+TSS_PCR_EVENT *concat_pcr_events(TSS_PCR_EVENT **, UINT32, TSS_PCR_EVENT *, UINT32);
+UINT32 get_pcr_event_size(TSS_PCR_EVENT *);
+void free_external_events(UINT32, TSS_PCR_EVENT *);
+
+extern struct event_log *tcs_event_log;
+
 #endif
