@@ -31,6 +31,7 @@
 #include "tcsd.h"
 #include "tcsd_ops.h"
 
+
 struct tcsd_config_options options_list[] = {
 	{"port", opt_port},
 	{"num_threads", opt_max_threads},
@@ -43,9 +44,21 @@ struct tcsd_config_options options_list[] = {
 	{"conformance_cred", opt_conformance_cred},
 	{"endorsement_cred", opt_endorsement_cred},
 	{"remote_ops", opt_remote_ops},
+	{"enforce_exclusive_transport", opt_exclusive_transport},
+	{"host_platform_class", opt_host_platform_class},
+	{"all_platform_classes", opt_all_platform_classes},
 	{NULL, 0}
 };
 
+struct tcg_platform_spec tcg_platform_specs[] = {
+	{"PC_11", TPM_PS_PC_11, TPM_PS_PC_11_URI},
+	{"PC_12", TPM_PS_PC_12, TPM_PS_PC_12_URI},
+	{"PDA_12", TPM_PS_PDA_12, TPM_PS_PDA_12_URI},
+	{"SERVER_12", TPM_PS_Server_12, TPM_PS_Server_12_URI},
+	{"MOBILE_12", TPM_PS_Mobile_12, TPM_PS_Mobile_12_URI},
+	{NULL, 0, 0}
+};
+
 
 void
 init_tcsd_config(struct tcsd_config *conf)
@@ -63,6 +76,55 @@ init_tcsd_config(struct tcsd_config *conf)
 	conf->endorsement_cred = NULL;
 	memset(conf->remote_ops, 0, sizeof(conf->remote_ops));
 	conf->unset = 0xffffffff;
+	conf->exclusive_transport = 0;
+	conf->host_platform_class = NULL;
+	conf->all_platform_classes = NULL;
+}
+
+TSS_RESULT
+platform_class_list_append(struct tcsd_config *conf, char *specName, TSS_BOOL is_main)
+{
+	int i;
+	struct platform_class *tmp, *new_class;
+
+	LogDebugFn("platform_class_list_append start:");
+	for (i = 0; tcg_platform_specs[i].name; i++) {
+		if (!strncasecmp(specName, tcg_platform_specs[i].name,
+				 strlen(tcg_platform_specs[i].name))) {
+			/* Allocate the new structure */
+			new_class = malloc(sizeof(struct platform_class));
+			if (new_class == NULL) {
+				LogError("malloc of %zd bytes failed",
+					 sizeof(struct platform_class));
+				return TCSERR(TSS_E_OUTOFMEMORY);
+			}
+			new_class->simpleID = tcg_platform_specs[i].specNo;
+			new_class->classURISize = strlen(tcg_platform_specs[i].specURI) + 1;
+			new_class->classURI = malloc(new_class->classURISize);
+			if (new_class->classURI == NULL) {
+				LogError("malloc of %u bytes failed", new_class->classURISize);
+				return TCSERR(TSS_E_OUTOFMEMORY);
+			}
+			memcpy(new_class->classURI, tcg_platform_specs[i].specURI,
+			       new_class->classURISize);
+
+			/* Append to the start of the list */
+			if (is_main) {
+				tmp = conf->host_platform_class;
+				conf->host_platform_class = new_class;
+			} else {
+				tmp = conf->all_platform_classes;
+				conf->all_platform_classes = new_class;
+			}
+			new_class->next = tmp;
+
+			LogDebugFn("Platform Class Added.");
+			return TSS_SUCCESS;
+		}
+	}
+
+	LogError("TCG Specification not supported: \"%s\"", specName);
+	return TCSERR(TSS_E_INTERNAL_ERROR);
 }
 
 void
@@ -92,6 +154,9 @@ config_set_defaults(struct tcsd_config *conf)
 
 	if (conf->unset & TCSD_OPTION_KERNEL_LOGFILE)
 		conf->kernel_log_file = strdup(TCSD_DEFAULT_KERNEL_LOG_FILE);
+
+	if (conf->unset & TCSD_OPTION_HOST_PLATFORM_CLASS)
+		platform_class_list_append(conf, "PC_12", TRUE);
 }
 
 int
@@ -197,6 +262,7 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 {
 	char *ptr = buf, *tmp_ptr = NULL, *arg, *comma;
 	int option, tmp_int;
+	TSS_RESULT result;
 
 	if (ptr == NULL || *ptr == '\0' || *ptr == '#' || *ptr == '\n')
 		return TSS_SUCCESS;
@@ -249,7 +315,8 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 					conf->firmware_pcrs |= (1 << tmp_int);
 				else
 					LogError("Config option \"firmware_pcrs\" is out of range."
-						 "%s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num, tmp_int);
+						 "%s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num,
+						 tmp_int);
 				break;
 			}
 
@@ -258,8 +325,8 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 			if (tmp_int >= 0 && tmp_int < TCSD_MAX_PCRS)
 				conf->firmware_pcrs |= (1 << tmp_int);
 			else
-				LogError("Config option \"firmware_pcrs\" is out of range. %s:%d: \"%d\"",
-						TCSD_CONFIG_FILE, line_num, tmp_int);
+				LogError("Config option \"firmware_pcrs\" is out of range. "
+					 "%s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num, tmp_int);
 		}
 		break;
 	case opt_kernel_pcrs:
@@ -277,7 +344,8 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 					conf->kernel_pcrs |= (1 << tmp_int);
 				else
 					LogError("Config option \"kernel_pcrs\" is out of range. "
-						 "%s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num, tmp_int);
+						 "%s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num,
+						 tmp_int);
 				break;
 			}
 
@@ -286,25 +354,25 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 			if (tmp_int >= 0 && tmp_int < TCSD_MAX_PCRS)
 				conf->kernel_pcrs |= (1 << tmp_int);
 			else
-				LogError("Config option \"kernel_pcrs\" is out of range. %s:%d: \"%d\"",
-						TCSD_CONFIG_FILE, line_num, tmp_int);
+				LogError("Config option \"kernel_pcrs\" is out of range. "
+					 "%s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num, tmp_int);
 		}
 		break;
 	case opt_system_ps_file:
 		if (*arg != '/') {
-			LogError("Config option \"system_ps_dir\" must be an absolute path name. %s:%d: \"%s\"",
-					TCSD_CONFIG_FILE, line_num, arg);
+			LogError("Config option \"system_ps_dir\" must be an absolute path name. "
+				 "%s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, arg);
 		} else {
 			char *dir_ptr;
 			int rc;
 
 			if ((rc = get_file_path(arg, &tmp_ptr)) < 0) {
-				LogError("Config option \"system_ps_file\" is invalid. %s:%d: \"%s\"",
-						TCSD_CONFIG_FILE, line_num, arg);
+				LogError("Config option \"system_ps_file\" is invalid."
+					 " %s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, arg);
 				return TCSERR(TSS_E_INTERNAL_ERROR);
 			} else if (rc > 0) {
-				LogError("Config option \"system_ps_file\" is invalid. %s:%d: \"%s\"",
-						TCSD_CONFIG_FILE, line_num, tmp_ptr);
+				LogError("Config option \"system_ps_file\" is invalid. %s:%d:"
+					 " \"%s\"", TCSD_CONFIG_FILE, line_num, tmp_ptr);
 				return TCSERR(TSS_E_INTERNAL_ERROR);
 			}
 			if (tmp_ptr == NULL)
@@ -335,18 +403,18 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 		break;
 	case opt_kernel_log:
 		if (*arg != '/') {
-			LogError("Config option \"kernel_log\" must be an absolute path name. %s:%d: \"%s\"",
-					TCSD_CONFIG_FILE, line_num, arg);
+			LogError("Config option \"kernel_log\" must be an absolute path name."
+				 " %s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, arg);
 		} else {
 			int rc;
 
 			if ((rc = get_file_path(arg, &tmp_ptr)) < 0) {
 				LogError("Config option \"kernel_log\" is invalid. %s:%d: \"%s\"",
-						TCSD_CONFIG_FILE, line_num, arg);
+					 TCSD_CONFIG_FILE, line_num, arg);
 				return TCSERR(TSS_E_INTERNAL_ERROR);
 			} else if (rc > 0) {
 				LogError("Config option \"kernel_log\" is invalid. %s:%d: \"%s\"",
-						TCSD_CONFIG_FILE, line_num, tmp_ptr);
+					 TCSD_CONFIG_FILE, line_num, tmp_ptr);
 				return TCSERR(TSS_E_INTERNAL_ERROR);
 			}
 			if (tmp_ptr == NULL)
@@ -361,18 +429,18 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 		break;
 	case opt_firmware_log:
 		if (*arg != '/') {
-			LogError("Config option \"firmware_log\" must be an absolute path name. %s:%d: \"%s\"",
-					TCSD_CONFIG_FILE, line_num, arg);
+			LogError("Config option \"firmware_log\" must be an absolute path name."
+				 " %s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, arg);
 		} else {
 			int rc;
 
 			if ((rc = get_file_path(arg, &tmp_ptr)) < 0) {
 				LogError("Config option \"firmware_log\" is invalid. %s:%d: \"%s\"",
-						TCSD_CONFIG_FILE, line_num, arg);
+					 TCSD_CONFIG_FILE, line_num, arg);
 				return TCSERR(TSS_E_INTERNAL_ERROR);
 			} else if (rc > 0) {
 				LogError("Config option \"firmware_log\" is invalid. %s:%d: \"%s\"",
-						TCSD_CONFIG_FILE, line_num, tmp_ptr);
+					 TCSD_CONFIG_FILE, line_num, tmp_ptr);
 				return TCSERR(TSS_E_INTERNAL_ERROR);
 			}
 			if (tmp_ptr == NULL)
@@ -476,7 +544,8 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 				if (comma != NULL) {
 					if (tcsd_set_remote_op(conf, comma)) {
 						LogError("Config option \"remote_ops\" is invalid. "
-								"%s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, comma);
+							 "%s:%d: \"%s\"", TCSD_CONFIG_FILE,
+							 line_num, comma);
 					}
 				}
 				break;
@@ -485,14 +554,77 @@ read_conf_line(char *buf, int line_num, struct tcsd_config *conf)
 			*comma++ = '\0';
 			if (tcsd_set_remote_op(conf, comma)) {
 				LogError("Config option \"remote_ops\" is invalid. "
-						"%s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, comma);
+					 "%s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, comma);
+			}
+		}
+		break;
+        case opt_exclusive_transport:
+		tmp_int = atoi(arg);
+		if (tmp_int < 0 || tmp_int > 1) {
+			LogError("Config option \"enforce_exclusive_transport\" out of range."
+				 " %s:%d: \"%d\"", TCSD_CONFIG_FILE, line_num, tmp_int);
+			return TCSERR(TSS_E_INTERNAL_ERROR);
+		} else {
+			conf->exclusive_transport = tmp_int;
+			conf->unset &= ~TCSD_OPTION_EXCLUSIVE_TRANSPORT;
+		}
+		break;
+	case opt_host_platform_class:
+		/* append the host class on the list */
+		conf->unset &= ~TCSD_OPTION_HOST_PLATFORM_CLASS;
+		comma = rindex(arg,'\n');
+		*comma = '\0';
+
+		comma = rindex(arg,',');
+		/* At least one comma: error - more than one host class defined */
+		if (comma != NULL) {
+			LogError("Config option \"host_platform_class\" error: more than one "
+				 "defined. %s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, comma);
+			return TCSERR(TSS_E_INTERNAL_ERROR);
+		} else {
+			comma = arg;
+			/* Add the platform class on the list */
+			if ((result = platform_class_list_append(conf, comma, TRUE))){
+				LogError("Config option \"host_platform_class\" invalid. "
+					 "%s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, comma);
+				return result;
+			}
+		}
+		break;
+	case opt_all_platform_classes:
+		/* append each of the comma separated values on the list */
+		comma = rindex(arg, '\n');
+		*comma = '\0';
+		while (1) {
+			comma = rindex(arg, ',');
+
+			if (comma == NULL) {
+				comma = arg;
+
+				if (comma != NULL) {
+					/* Add the platform class on the list */
+					if ((result = platform_class_list_append(conf, comma,
+										 FALSE))) {
+						LogError("Config option \"all_platform_class\" "
+							 "invalid. %s:%d: \"%s\"", TCSD_CONFIG_FILE,
+							 line_num, comma);
+						return result;
+					}
+				}
+				break;
+			}
+			*comma++ = '\0';
+			/* Add the platform class on the list */
+			if ((result = platform_class_list_append(conf, comma, FALSE))) {
+				LogError("Config option \"all_platform_class\" invalid. "
+					 "%s:%d: \"%s\"", TCSD_CONFIG_FILE, line_num, comma);
+				return result;
 			}
 		}
 		break;
 	default:
 		/* bail out on any unknown option */
-		LogError("Unknown config option %s:%d \"%s\"!",
-				TCSD_CONFIG_FILE, line_num, arg);
+		LogError("Unknown config option %s:%d \"%s\"!", TCSD_CONFIG_FILE, line_num, arg);
 		return TCSERR(TSS_E_INTERNAL_ERROR);
 	}
 
@@ -514,6 +646,20 @@ read_conf_file(FILE *f, struct tcsd_config *conf)
 	return TSS_SUCCESS;
 }
 
+void
+free_platform_lists(struct platform_class *list)
+{
+	struct platform_class *tmp;
+
+	while (list != NULL){
+		if (list->classURISize > 0)
+			free(list->classURI);
+		tmp = list->next;
+		free(list);
+		list = tmp;
+	}
+}
+
 void
 conf_file_final(struct tcsd_config *conf)
 {
@@ -524,6 +670,8 @@ conf_file_final(struct tcsd_config *conf)
 	free(conf->platform_cred);
 	free(conf->conformance_cred);
 	free(conf->endorsement_cred);
+	free_platform_lists(conf->host_platform_class);
+	free_platform_lists(conf->all_platform_classes);
 }
 
 TSS_RESULT
@@ -603,7 +751,6 @@ conf_file_init(struct tcsd_config *conf)
 	return result;
 }
 
-
 TSS_RESULT
 ps_dirs_init()
 {
