@@ -18,7 +18,6 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-#include <pthread.h>
 #include <assert.h>
 
 #include "trousers/tss.h"
@@ -30,7 +29,7 @@
 #include "tsplog.h"
 
 static int user_ps_fd = -1;
-static pthread_mutex_t user_ps_lock = PTHREAD_MUTEX_INITIALIZER;
+static MUTEX_DECLARE_INIT(user_ps_lock);
 
 /*
  * Determine the default path to the persistent storage file and create it if it doesn't exist.
@@ -117,13 +116,13 @@ get_file(int *fd)
 	int rc = 0;
 	char *file_name = NULL;
 
-	pthread_mutex_lock(&user_ps_lock);
+	MUTEX_LOCK(user_ps_lock);
 
 	/* check the global file handle first.  If it exists, lock it and return */
 	if (user_ps_fd != -1) {
 		if ((rc = flock(user_ps_fd, LOCK_EX))) {
 			LogDebug("USER PS: failed to lock file: %s", strerror(errno));
-			pthread_mutex_unlock(&user_ps_lock);
+			MUTEX_UNLOCK(user_ps_lock);
 			return TSPERR(TSS_E_INTERNAL_ERROR);
 		}
 
@@ -134,7 +133,7 @@ get_file(int *fd)
 	/* open and lock the file */
 	if ((result = get_user_ps_path(&file_name))) {
 		LogDebugFn("USER PS: error getting file path");
-		pthread_mutex_unlock(&user_ps_lock);
+		MUTEX_UNLOCK(user_ps_lock);
 		return result;
 	}
 
@@ -142,7 +141,7 @@ get_file(int *fd)
 	if (user_ps_fd < 0) {
 		LogDebug("USER PS: open of %s failed: %s", file_name, strerror(errno));
 		free(file_name);
-		pthread_mutex_unlock(&user_ps_lock);
+		MUTEX_UNLOCK(user_ps_lock);
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
 
@@ -151,7 +150,7 @@ get_file(int *fd)
 		free(file_name);
 		close(user_ps_fd);
 		user_ps_fd = -1;
-		pthread_mutex_unlock(&user_ps_lock);
+		MUTEX_UNLOCK(user_ps_lock);
 		return TSPERR(TSS_E_INTERNAL_ERROR);
 	}
 
@@ -173,7 +172,7 @@ put_file(int fd)
 		rc = -1;
 	}
 
-	pthread_mutex_unlock(&user_ps_lock);
+	MUTEX_UNLOCK(user_ps_lock);
 	return rc;
 }
 
@@ -182,7 +181,7 @@ psfile_close(int fd)
 {
 	close(fd);
 	user_ps_fd = -1;
-	pthread_mutex_unlock(&user_ps_lock);
+	MUTEX_UNLOCK(user_ps_lock);
 }
 
 TSS_RESULT
@@ -415,10 +414,10 @@ psfile_write_key(int fd,
 		 UINT16 key_blob_size)
 {
 	TSS_RESULT result;
-	TCPA_KEY key;
+	TSS_KEY key;
 	UINT32 zero = 0;
-	UINT16 pub_key_size, cache_flags = 0;
 	UINT64 offset;
+	UINT16 pub_key_size, cache_flags = 0;
 	struct stat stat_buf;
 	int rc, file_offset;
 
@@ -447,7 +446,7 @@ psfile_write_key(int fd,
 
 	/* Unload the blob to get the public key */
 	offset = 0;
-	if ((result = Trspi_UnloadBlob_KEY(&offset, key_blob, &key)))
+	if ((result = UnloadBlob_TSS_KEY(&offset, key_blob, &key)))
 		return result;
 
 	pub_key_size = key.pubKey.keyLength;
@@ -500,7 +499,7 @@ psfile_write_key(int fd,
 		goto done;
 	}
 
-	if ((result = psfile_change_num_keys(fd, INCREMENT))) {
+	if ((result = psfile_change_num_keys(fd, TSS_PSFILE_INCREMENT_NUM_KEYS))) {
 		LogDebug("%s", __FUNCTION__);
 		goto done;
 	}
@@ -592,7 +591,7 @@ psfile_remove_key(int fd, TSS_UUID *uuid)
 
 	/* we succeeded in removing a key from the disk. Decrement the number
 	 * of keys in the file */
-	if ((result = psfile_change_num_keys(fd, DECREMENT)))
+	if ((result = psfile_change_num_keys(fd, TSS_PSFILE_DECREMENT_NUM_KEYS)))
 		return result;
 
 	return TSS_SUCCESS;
@@ -707,7 +706,7 @@ err_exit:
 TSS_RESULT
 copy_key_info(int fd, TSS_KM_KEYINFO *ki, struct key_disk_cache *c)
 {
-	TCPA_KEY key;
+	TSS_KEY key;
 	BYTE blob[4096];
 	UINT64 offset;
 	TSS_RESULT result;
@@ -728,10 +727,16 @@ copy_key_info(int fd, TSS_KM_KEYINFO *ki, struct key_disk_cache *c)
 
 	/* Expand the blob into a useable form */
 	offset = 0;
-	if ((result = Trspi_UnloadBlob_KEY(&offset, blob, &key)))
+	if ((result = UnloadBlob_TSS_KEY(&offset, blob, &key)))
 		return result;
 
-	memcpy(&ki->versionInfo, &key.ver, sizeof(TSS_VERSION));
+	if (key.hdr.key12.tag == TPM_TAG_KEY12) {
+		ki->versionInfo.bMajor = TSS_SPEC_MAJOR;
+		ki->versionInfo.bMinor = TSS_SPEC_MINOR;
+		ki->versionInfo.bRevMajor = 0;
+		ki->versionInfo.bRevMinor = 0;
+	} else
+		memcpy(&ki->versionInfo, &key.hdr.key11.ver, sizeof(TSS_VERSION));
 	memcpy(&ki->keyUUID, &c->uuid, sizeof(TSS_UUID));
 	memcpy(&ki->parentKeyUUID, &c->parent_uuid, sizeof(TSS_UUID));
 	ki->bAuthDataUsage = key.authDataUsage;
@@ -741,6 +746,56 @@ copy_key_info(int fd, TSS_KM_KEYINFO *ki, struct key_disk_cache *c)
 	return TSS_SUCCESS;
 }
 
+TSS_RESULT
+copy_key_info2(int fd, TSS_KM_KEYINFO2 *ki, struct key_disk_cache *c)
+{
+	TSS_KEY key;
+	BYTE blob[4096];
+	UINT64 offset;
+	TSS_RESULT result;
+	off_t off;
+
+	/* Set the file pointer to the offset that the key blob is at */
+	off = lseek(fd, TSSPS_BLOB_DATA_OFFSET(c), SEEK_SET);
+	if (off == ((off_t)-1)) {
+		LogDebug("lseek: %s", strerror(errno));
+		return TSPERR(TSS_E_INTERNAL_ERROR);
+	}
+
+	/* Read in the key blob */
+	if ((result = read_data(fd, (void *)blob, c->blob_size))) {
+		LogDebug("%s", __FUNCTION__);
+		return result;
+	}
+
+	/* Expand the blob into a useable form */
+	offset = 0;
+	if ((result = UnloadBlob_TSS_KEY(&offset, blob, &key)))
+		return result;
+
+	if (key.hdr.key12.tag == TPM_TAG_KEY12) {
+		ki->versionInfo.bMajor = TSS_SPEC_MAJOR;
+		ki->versionInfo.bMinor = TSS_SPEC_MINOR;
+		ki->versionInfo.bRevMajor = 0;
+		ki->versionInfo.bRevMinor = 0;
+	} else
+		memcpy(&ki->versionInfo, &key.hdr.key11.ver, sizeof(TSS_VERSION));
+	memcpy(&ki->keyUUID, &c->uuid, sizeof(TSS_UUID));
+	memcpy(&ki->parentKeyUUID, &c->parent_uuid, sizeof(TSS_UUID));
+
+	/* CHECK: fill the two new fields of TSS_KM_KEYINFO2 */
+	ki->persistentStorageType = TSS_PS_TYPE_USER;
+	ki->persistentStorageTypeParent = c->flags & CACHE_FLAG_PARENT_PS_SYSTEM ?
+					  TSS_PS_TYPE_SYSTEM : TSS_PS_TYPE_USER;
+
+	ki->bAuthDataUsage = key.authDataUsage;
+
+	free_key_refs(&key);
+
+	return TSS_SUCCESS;
+}
+
+
 TSS_RESULT
 psfile_get_registered_keys(int fd,
 			   TSS_UUID *uuid,
@@ -828,6 +883,95 @@ restart_search:
 	return TSS_SUCCESS;
 }
 
+TSS_RESULT
+psfile_get_registered_keys2(int fd,
+			   TSS_UUID *uuid,
+			   TSS_UUID *tcs_uuid,
+			   UINT32 *size,
+			   TSS_KM_KEYINFO2 **keys)
+{
+	TSS_RESULT result;
+	struct key_disk_cache *cache_entries;
+	UINT32 cache_size, i, j;
+	TSS_KM_KEYINFO2 *keyinfos = NULL;
+	TSS_UUID *find_uuid;
+
+        if ((result = psfile_get_all_cache_entries(fd, &cache_size, &cache_entries)))
+                return result;
+
+	if (cache_size == 0) {
+		if (uuid)
+			return TSPERR(TSS_E_PS_KEY_NOTFOUND);
+		else {
+			*size = 0;
+			*keys = NULL;
+			return TSS_SUCCESS;
+		}
+	}
+
+	if (uuid) {
+		find_uuid = uuid;
+		j = 0;
+
+		restart_search:
+			/* Search for the requested UUID.  When found, allocate new space for it, copy
+			 * it in, then change the uuid to be searched for it its parent and start over. */
+			for (i = 0; i < cache_size; i++) {
+				/*Return 0 if normal finish*/
+				if (!memcmp(&cache_entries[i].uuid, find_uuid, sizeof(TSS_UUID))) {
+					if (!(keyinfos = realloc(keyinfos,
+							(j+1) * sizeof(TSS_KM_KEYINFO2)))) {
+						free(cache_entries);
+						free(keyinfos);
+						return TSPERR(TSS_E_OUTOFMEMORY);
+					}
+					/* Here the key UUID is found and needs to be copied for the array*/
+					/* Initializes the keyinfos with 0's*/
+					memset(&keyinfos[j], 0, sizeof(TSS_KM_KEYINFO2));
+
+					if ((result = copy_key_info2(fd, &keyinfos[j], &cache_entries[i]))) {
+						free(cache_entries);
+						free(keyinfos);
+						return result;
+					}
+
+					find_uuid = &keyinfos[j].parentKeyUUID;
+					j++;
+					goto restart_search;
+				}
+			}
+
+		/* Searching for keys in the user PS will always lead us up to some key in the
+		 * system PS. Return that key's uuid so that the upper layers can call down to TCS
+		 * to search for it. */
+		memcpy(tcs_uuid, find_uuid, sizeof(TSS_UUID));
+
+		*size = j;
+	} else {
+		if ((keyinfos = calloc(cache_size, sizeof(TSS_KM_KEYINFO2))) == NULL) {
+			LogDebug("malloc of %zu bytes failed.",
+					cache_size * sizeof(TSS_KM_KEYINFO2));
+			free(cache_entries);
+			return TSPERR(TSS_E_OUTOFMEMORY);
+		}
+
+		for (i = 0; i < cache_size; i++) {
+			if ((result = copy_key_info2(fd, &keyinfos[i], &cache_entries[i]))) {
+				free(cache_entries);
+				free(keyinfos);
+				return result;
+			}
+		}
+
+		*size = cache_size;
+	}
+
+	free(cache_entries);
+
+	*keys = keyinfos;
+
+	return TSS_SUCCESS;
+}
 
 /*
  * read into the PS file and return the number of keys
